@@ -18,6 +18,7 @@ logger = logging.getLogger(__name__)
 class EveItem(EveModelBase):
     session = param.ClassSelector(EveSessionBase, constant=True, precedence=-1)
     _resource_url = param.String(precedence=-1)
+    _id = param.String(allow_None=False, precedence=1)
     _etag = param.String(allow_None=True, precedence=15)
     _version = param.Integer(default=1,
                              bounds=(1, None),
@@ -86,11 +87,12 @@ class EveItem(EveModelBase):
                 class_ = param.Selector
                 kwargs["objects"] = field_schema["allowed"]
 
-            elif field_schema["type"] in TYPE_MAPPING:
+            elif field_schema.get("type", 'string') in TYPE_MAPPING:
                 class_ = EveField(extended_name, field_schema,
-                                  TYPE_MAPPING[field_schema["type"]])
+                                  TYPE_MAPPING[field_schema.get("type", 'string')])
             else:
-                continue
+                class_ = param.Parameter
+
             if "default" in field_schema:
                 kwargs["default"] = field_schema["default"]
             else:
@@ -100,8 +102,7 @@ class EveItem(EveModelBase):
             if widget is not None:
                 _widgets[field_name] = widget
 
-            if field_schema.get("required", False):
-                kwargs["allow_None"] = False
+            kwargs["allow_None"] = field_schema.get("nullable", True)
 
             bounds = (field_schema.get("min",
                                        None), field_schema.get("max", None))
@@ -182,7 +183,7 @@ class EveItem(EveModelBase):
                     if param.constant or param.readonly:
                         setattr(self, getattr(param, "_internal_name"), v)
                     else:
-                        self.param.set_param(k=v)
+                        self.param.set_param(**{k:v})
                 except Exception as e:
                     logger.error(str(e))
                     pass
@@ -221,21 +222,21 @@ class EveItem(EveModelBase):
         ]
 
     def push(self):
-        headers ={}
+        headers = {"Content-Type": "application/json"}
         if self._version == self._latest_version:
             headers["If-Match"] = self._etag
         data = {k: getattr(self, k) for k in self.schema if not k.startswith("_")}
         doc = {k:v for k,v in data.items() if v is not None}
         files = {name: BytesIO(doc.pop(name)) for name, value in data.items()
                     if isinstance(value, bytes)}
-        data = to_data_dict(doc)
-        
-        with self.session.Client() as client:
-            resp = client.put(self.url, data=data, files=files, headers=headers)
+        # data = to_data_dict(doc)
+        data = json.dumps(doc, cls=NumpyJSONENncoder)
+        with self.session.Client(headers=headers) as client:
+            resp = client.put(self.url, data=data, files=files, )
         self.pull()
 
     def patch(self, *fields):
-        headers ={}
+        headers = {"Content-Type": "application/json"}
         if self._version == self._latest_version:
             headers["If-Match"] = self._etag
             
@@ -244,7 +245,8 @@ class EveItem(EveModelBase):
             data[k] = getattr(self, k)
         doc = {k:v for k,v in data.items() if v is not None}
         files = {name: BytesIO(doc.pop(name)) for name, value in data.items() if isinstance(value, bytes)}
-        data = to_data_dict(doc)
+        # data = to_data_dict(doc)
+        data = json.dumps(doc, cls=NumpyJSONENncoder)
 
         with self.session.Client() as client:
             resp = client.patch(self.url, data=data, files=files, headers=headers)
